
#pragma once

#include <cassert>
#include <iostream>



class OBitstream
{
public:
  OBitstream( std::ostream& os )
    : outStream( os )
    , nextByte ( 0  )
    , freeBits ( 8  )
  {}
  ~OBitstream()
  {
    flush();
  }

  bool good() const
  {
    return outStream.good();
  }
  
  void addBit( bool bit )
  {
    nextByte = ( nextByte << 1 ) + uint8_t( bit );
    if( ! --freeBits )
    {
      outStream.put( nextByte );
      nextByte = 0;
      freeBits = 8;
    }
  }

  template< typename T >
  void addFixed( T value, unsigned numBits )
  {
    assert( numBits <= ( sizeof(T) << 3 ) );
    while( numBits-- )
      addBit( ( value >> numBits ) & T(1) );
  }
  
  void flush()
  {
    while( freeBits < 8 )
      addBit( 0 );
    outStream.flush();
  }
  
private:
  std::ostream&  outStream;
  uint8_t        nextByte;
  uint8_t        freeBits;
};





class IBitstream
{
public:
  IBitstream ( std::istream& is )
    : inStream ( is )
    , currByte ( 0  )
    , availBits( 0  )
    , errBytes ( 0  )
  {}
  
  ~IBitstream()
  {}

  bool good() const
  {
    return inStream.good();
  }
  
  bool getBit()
  {
    if( !availBits )
    {
      if( !errBytes )
      {
		int byte = inStream.get();
		if( byte != std::char_traits<char>::eof() )
			currByte = uint8_t(byte);
		else {
			currByte = 0;
			errBytes = 1;
		}
      }
      else
      {
		errBytes++;
      }
      availBits = 8;
    }
    return ( ( currByte >> --availBits ) & 1 );
  }
  
  template< typename T >
  T getFixed( unsigned numBits )
  {
    assert( numBits <= ( sizeof(T) << 3 ) );
    T value = T(0);
    while( numBits-- )
      value = ( value << 1 ) + (T)getBit();
    return value;
  }

private:
  std::istream&  inStream;
  uint8_t        currByte;
  uint8_t        availBits;
  size_t         errBytes;
};


