
#include <string>
#include <fstream>
#include <iostream>
#include <vector>
#include <map>

#include "bitstream.h"
#include "arithCoding.h"

using namespace std;


//====== Pmf implementation (for toy example) =====
class Pmf : public IPmf
{
public:
  Pmf()
    : V  ( 4 )
    , pv { 8, 5, 3 }
  {}
  uint64_t operator[] ( unsigned index ) const  { return pv[ index ]; }
  void     update     ( unsigned index ) {}

private:
  const unsigned   V;
  vector<uint64_t> pv;
};


//====== main =====
int main( int argc, char** argv )
{
  const unsigned U = 16;
  const unsigned V = 4;  // only possibility for our fixed pmf (in toy example)
  const string   bitstreamFile = "test.bin";

  const string   orgMessage = "BANANA";
  const string   alphabet   = "ANB";    // sorted
  const unsigned numSymbols = (unsigned)orgMessage.size();
  
  //----- encoding -----
  {
    ofstream   os( bitstreamFile );
    OBitstream bs( os );     
    assert( bs.good() );

    map<char,unsigned> char2index;
    for( unsigned k = 0; k < (unsigned)alphabet.size(); k++ )
      char2index[ alphabet[ k ] ] = k;

    Pmf        pmf;
    ArithEnc   aenc( U, V, bs );
    for( char c : orgMessage ) {
      aenc.encode( char2index.at( c ), pmf );
    }
    aenc.terminate();
    bs.flush();
  }

  //----- decoding -----
  {
    string     recMessage;
    ifstream   is( bitstreamFile );
    IBitstream bs( is );     
    assert( bs.good() );

    Pmf        pmf;
    ArithDec   adec( U, V, bs );
    for( unsigned k = 0; k < numSymbols; k++ ) {
      recMessage += alphabet[ adec.decode( pmf ) ];
    }
    cout << "reconstructed message = \"" << recMessage << "\""
	 << "  [" << ( recMessage == orgMessage ? "correct" : "wrong" ) << "]" << endl;
  }

  return 0;
}

